#include <Arduino.h>
#include <U8g2lib.h> //Use U8g2 library
#include <time.h>

// Determine whether to use SPI or I2C protocol
#ifdef U8X8_HAVE_HW_SPI
#include <SPI.h>
#endif
#ifdef U8X8_HAVE_HW_I2C
#include <Wire.h>
#endif

#define ButnPin D1 //Defined for a XIAO ESPC3 plugged into the Xiao Starter Kit, you may need to change this if not using the Starter Kit or a different controller
bool timeAmPm = true;

U8G2_SSD1306_128X64_NONAME_F_HW_I2C oled(U8G2_R0, /* reset=*/U8X8_PIN_NONE);

#include <PCF8563.h> //Declare library file for RTC chip
PCF8563 rtc; 

#include <WiFi.h>

const char* ssid     = "Gast"; //Replace with your SSID
const char* password = "LollyLegs!"; //TReplace with your password
const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = -25200; //-7 hours MST
const int   daylightOffset_sec = 3600; //1 hour DST

struct tm timeinfo;
struct timeDisplay
{
  char sdate[16];
  char stime12[16];
  char stime24[16];
};

timeDisplay td;

void setup()
{
  pinMode(ButnPin, INPUT_PULLUP);
  oled.begin();           // Initialize the OLED library
  oled.enableUTF8Print(); // enable UTF8 support for the Arduino print() function
  oled.setFont(u8g2_font_ncenB12_tr);
  oled.setFontDirection(0);
  oled.clearBuffer();
 
  Wire.begin(); // Wire init, adding the I2C bus.
  WiFi.begin(ssid, password);
  int count = 0;
  while ( WiFi.status() != WL_CONNECTED )
  {
    count++;
    delay(500);
    
    if (count > 30)
    {
      oled.println("Wifi Timeout.");
      oled.sendBuffer();
      while (true);
    }
    
  }
  // Set the current date and time. After setting, it will start timing from this moment
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  if(!getLocalTime(&timeinfo))
  {
    oled.println("Time Failure.");
    while (true);
  }
  //disconnect WiFi as it's no longer needed
  WiFi.disconnect(true);
  WiFi.mode(WIFI_OFF);

  rtc.setYear(timeinfo.tm_year);  // Year
  rtc.setMonth(timeinfo.tm_mon);  // Month
  rtc.setDay(timeinfo.tm_mday);   // Day
  rtc.setHour(timeinfo.tm_hour);  // Hour
  rtc.setMinut(timeinfo.tm_min); // Minute
  rtc.setSecond(timeinfo.tm_sec); // Second

  rtc.startClock(); // Clock starts timing
}

void getDisplayTime(Time t, timeDisplay *td)
{
  sprintf(td->sdate, "%02d/%02d/%i", t.day, t.month+1, t.year + 1900);
  sprintf(td->stime12, "%02d:%02d:%02d%s", t.hour > 12 ? t.hour - 12 : t.hour, t.minute, t.second, t.hour > 12 ? " PM" : " AM");
  sprintf(td->stime24, "%02d:%02d:%02d", t.hour, t.minute, t.second);
}

void loop()
{
  if (digitalRead(ButnPin) == LOW)
    timeAmPm = !timeAmPm;
  Time nowTime = rtc.getTime(); //Get time
  getDisplayTime(nowTime, &td);
  oled.clearBuffer();
  //Print the current date and time on the oled screen
  oled.setCursor(24, 15);
  oled.println(td.sdate);
  oled.setCursor(24, 32);
  //oled.setCursor(24, 49); optional if wanting to skip a line
  if (timeAmPm)
    oled.println(td.stime12);
  else
    oled.println(td.stime24);
  oled.sendBuffer();
  delay(1000);
}
